function [w0, z0, z1, z2, g12] = Gaussian_Resonator (R1, R2, n, L, lambda0, nfig)

% ---------------------------------------------------------------------------------------
%    [w0, z0, z1, z2, g12] = Gaussian_Resonator (R1, R2, n, L, lambda0, nfig)
%    Example:  [w0, z0, z1, z2, g12] = Gaussian_Resonator (0.3, 0.5, 1, 0.6, 0.6328, 11)
%
%    INPUTS
%    R1       = Radius of curvature of left mirror in meters (positive or negative)
%    R2       = Radius of curvature of right mirror in meters (positive or negative)
%    n        = refractive index of cavity medium
%    L        = Cavity length in meters
%    lambda0  = freespace wavelength in microns
%    nfig     = initial figure
%
%    OUTPUTS
%    w0       = minimum waist in microns
%    z0       = Rayleigh distance in meters
%    z1       = distance of left mirror from Gaussian zero (in meters)
%    z2       = distance of right mirror from Gaussian zero (in meters)
%    g12      = Stability parameter
%
% ---------------------------------------------------------------------------------------
 
%
%    Output variables Initialization (to NaN)
%

   w0  = NaN;
   z0  = NaN;
   z1  = NaN;
   z2  = NaN;
   g12 = NaN; 
 
   if R1>0 & R2>0
      Icase = 1;
   elseif R1<0 & R2>0
      Icase = 2;
   elseif R1>0 & R2<0
      Icase = 3;
   elseif R1<0 & R2<0
      Icase = 4;
   end
   
   if Icase == 4
      disp ('----   Unstable resonator   ----')
      disp ('---- Program will terminate ----')
      return
   end
 
   Icase
   g1  = 1 - (L/R1);
   g2  = 1 - (L/R2);
   g12  = g1*g2; 
 
   if (g12 < 0) | (g12 > 1)
      disp ('----   Unstable resonator   ----')
      disp ('---- Program will terminate ----')
      fprintf(' g1*g2 = %d\n',g12)
      return
   end
 
   Npoints = 2000;
   
   if g12 >= 0 & abs(R1+R2-2*L)>10*eps
     z02 = L*(R1-L)*(R2-L)*(R1+R2-L)/ (R1+R2-2*L)^2 ;
     z0  = z02^(1/2);
     w0  = ((z0*lambda0*10^(-6))/(pi*n))^(1/2);
     z1  = L*(R2-L)/(R1+R2-2*L);
     z1  = -z1;
     z2  = L*(R1-L)/(R1+R2-2*L);
     g1  = 1 - (L/R1);
     g2  = 1 - (L/R2);
     g12  = g1*g2;
   elseif g12<10*eps & abs(R1-R2)<10*eps
     z02 = L*(R1+R2-L)/4;
     z0  = z02^(1/2);
     w0  = ((z0*lambda0*10^(-6))/(pi*n))^(1/2);
     z1  = L/2;
     z1  = -z1;
     z2  = L/2;
     g1  = 1 - (L/R1);
     g2  = 1 - (L/R2);
     g12  = g1*g2;
   elseif  abs(R1+R2-2*L)<10*eps
     disp(' ---- Case should not be possible ---- ')
     fprintf('R1 = %d, R2 = %d, L = %d\n',R1,R2)
     return
   end   
   
   
   
   if abs( imag(z0) ) >10*eps
      disp ('----   Unstable resonator   ----')
      disp ('---- Program will terminate ----')
      return
   end
   
   zz=z2-z1;
   if abs(zz-L) > 10*eps
      disp('----- Warning -----')
      zz=zz
      L=L
   end
   
   zz = z1:L/Npoints:z2;
   
   w1 = w0*(1+(z1/z0).^2).^(1/2)
   w2 = w0*(1+(z2/z0).^2).^(1/2)
   wz = w0*(1+(zz/z0).^2).^(1/2);
   
   
   figure(nfig)
   hold off
   
   plot(100*zz,wz*1e6,'r',100*zz,-wz*1e6,'r','Linewidth',2)
   hold on
   plot([100*z1,100*z2],[0,0],'b','Linewidth',2)
     
   if Icase == 1
   
     theta1a = atan2(w1,z1)
     theta1b = 2*pi-theta1a
     phi = theta1a:(theta1b-theta1a)/Npoints:theta1b;
     if abs(z1) > 100*eps
       zz1 = R1+z1+abs(R1)*cos(phi);
       RR1 = abs(R1)*sin(phi);
     else
       zz1(1) = z1;
       zz1(2) = z1;
       RR1(1) = -2.5*w1;
       RR1(2) = +2.5*w1;
     end
   
     theta1a = atan2(w2,z2)
     theta1b = -theta1a
     phi = theta1b:(theta1a-theta1b)/Npoints:theta1a;
     zz2 = (z2-R2) + abs(R2)*cos(phi);
     RR2 = abs(R2)*sin(phi);
     
   elseif Icase == 2
   
     theta1a = atan2(w1,z1)
     theta1b = -theta1a
     phi = theta1b:(theta1a-theta1b)/Npoints:theta1a;
     zz1 = z1+R1+abs(R1)*cos(phi);
     RR1 = abs(R1)*sin(phi);
     zz2 = z2-R2+abs(R2)*cos(phi);
     RR2 = abs(R2)*sin(phi);
     
   elseif Icase == 3
   
     theta1a = atan2(w1,z1)
     theta1b = 2*pi-theta1a
     phi = theta1b:(theta1a-theta1b)/Npoints:theta1a;
     zz1 = z1+R1+abs(R1)*cos(phi);
     RR1 = abs(R1)*sin(phi);
     zz2 = z2-R2+abs(R2)*cos(phi);
     RR2 = abs(R2)*sin(phi);
   
   end
   
   plot(100*zz1,1e6*RR1,'--m',100*zz2,1e6*RR2,'--m','Linewidth',2)
    
   set(gca,'Fontsize',12,'Fontweight','bold')

   xlabel('Distance z  (cm)','Fontsize',12)
   ylabel('Beam Waist, w(z) (\mum)','Fontsize',12)
   title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, L = ',num2str(L),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
   grid on