function [w0, z0, z1, z2, L, g12] = Stable_Resonator_Gaussian (R1, R2, n, lambda0, nfig)

% ---------------------------------------------------------------------------------------
%
%  [w0, z0, z1,z2] = Stable_Resonator_Gaussian (R1, R2, n, lambda0, nfig)
%  
%  Example:  [w0, z0, z1,z2] = Stable_Resonator_Gaussian (0.3, 0.5, 1, 0.6328, 1)
%
%    INPUTS
%    R1       = Radius of curvature of left mirror in meters (positive or negative)
%    R2       = Radius of curvature of right mirror in meters (positive or negative)
%    n        = refractive index of cavity medium
%    lambda0  = freespace wavelength in microns
%    nfig     = initial figure
%
%    OUTPUTS
%    w0       = minimum waist in microns
%    z0       = Rayleigh distance in meters
%    z1       = distance of left mirror from Gaussian zero (in meters)
%    z2       = distance of right mirror from Gaussian zero (in meters)
%    L        = Cavity length in meters
%    g12      = Stability parameter
%
% ---------------------------------------------------------------------------------------

%
%    Output variables Initialization (to NaN)
%

w0  = NaN;
z0  = NaN;
z1  = NaN;
z2  = NaN;
L   = NaN;
g12 = NaN;


rho(1) = 0;
rho(2) = R1;
rho(3) = R2;
rho(4) = R1+R2;

rho     =  sort(rho,'ascend');
Npoints = 2000;

iL1 = 0;
iL2 = 0;
Delta_L1 = rho(2)-rho(1);
Delta_L2 = rho(4)-rho(3);

if Delta_L1 > 10*eps
   iL1 = 1;
   L1 = rho(1):Delta_L1/Npoints:rho(2);
end

if Delta_L2 > 10*eps
   iL2 = 1;
   L2 = rho(3):Delta_L2/Npoints:rho(4);
end

if iL1 == 1 & iL2 == 1

   z02 = L1.*(R1-L1).*(R2-L1).*(R1+R2-L1)./ (R1+R2-2*L1).^2 ;
   z0a  = z02.^(1/2);
   w0a  = ((z0a*lambda0*10^(-6))/(pi*n)).^(1/2);
   z1a  = L1.*(R2-L1)./(R1+R2-2*L1);
   z1a  = -z1a;
   z2a  = L1.*(R1-L1)./(R1+R2-2*L1);
   g1a  = 1 - (L1/R1);
   g2a  = 1 - (L1/R2);
   g12a = g1a.*g2a;

   z02 = L2.*(R1-L2).*(R2-L2).*(R1+R2-L2)./ (R1+R2-2*L2).^2 ;
   z0b  = z02.^(1/2);
   w0b  = ((z0b*lambda0*10^(-6))/(pi*n)).^(1/2);
   z1b  = L2.*(R2-L2)./(R1+R2-2*L2);
   z1b  = -z1b;
   z2b  = L2.*(R1-L2)./(R1+R2-2*L2);
   g1b  = 1 - (L2/R1);
   g2b  = 1 - (L2/R2);
   g12b = g1b.*g2b;

   z0  = [z0a z0b];
   w0  = [w0a w0b];
   z1  = [z1a z1b];
   z1  = -z1;
   z2  = [z2a z2b];
   g12 = [g12a g12b];
   L  = [L1 L2];
   
elseif  iL1 == 0 & iL2 == 1

   z02 = L2.*(R1-L2).*(R2-L2).*(R1+R2-L2)./ (R1+R2-2*L2).^2 ;
   z0  = z02.^(1/2);
   w0  = ((z0*lambda0*10^(-6))/(pi*n)).^(1/2);
   z1  = L2.*(R2-L2)./(R1+R2-2*L2);
   z1  = -z1;
   z2  = L2.*(R1-L2)./(R1+R2-2*L2);
   g1  = 1 - (L2/R1);
   g2  = 1 - (L2/R2);
   g12  = g1.*g2;
   L = L2;
   
elseif iL1==1 & iL2 == 0

   z02 = L1.*(R1-L1).*(R2-L1).*(R1+R2-L1)./ (R1+R2-2*L1).^2 ;
   z0  = z02.^(1/2);
   w0  = ((z0*lambda0*10^(-6))/(pi*n)).^(1/2);
   z1  = L1.*(R2-L1)./(R1+R2-2*L1);
   z1  = -z1;
   z2  = L1.*(R1-L1)./(R1+R2-2*L1);
   g1  = 1 - (L1/R1);
   g2  = 1 - (L1/R2);
   g12  = g1.*g2;
   L = L1;
   
elseif iL1 == 0 & iL2 == 0
   disp('----- Check radius R1 and R2 ------')
   disp('----- Not usable L !         ------')
end


if iL1 == 1 & iL2 == 1
figure(nfig)

plot(L1*100,w0a*10^6,'b',L2*100,w0b*10^6,'b','Linewidth',2)
set(gca,'Fontsize',12,'Fontweight','bold')
xlim([0 Inf])
xlabel('Cavity Length L (cm)','Fontsize',12)
ylabel('Minimum Waist, w_0 (\mum)','Fontsize',12)
title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
grid on 

nfig = nfig + 1;
figure(nfig)

plot(L1*100,z0a*100,'b',L2*100,z0b*100,'b','Linewidth',2)
set(gca,'Fontsize',12,'Fontweight','bold')
xlim([0 Inf])
xlabel('Cavity Length L (cm)','Fontsize',12)
ylabel('Rayleigh Distance, z_0 (cm)','Fontsize',12)
title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
grid on 

nfig = nfig + 1;
figure(nfig)

plot(L1*100,g12a,'b',L2*100,g12b,'b','Linewidth',2)
set(gca,'Fontsize',12,'Fontweight','bold')
xlim([0 Inf])
xlabel('Cavity Length L (cm)','Fontsize',12)
ylabel('Stability Term, g_1g_2','Fontsize',12)
title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
grid on 

nfig = nfig + 1;
figure(nfig)
hold off

ih1 = plot(L1*100,z1a*100,'b','Linewidth',2);
hold on
ih2 = plot(L1*100,z2a*100,'r','Linewidth',2);
ih3 = plot(L2*100,z1b*100,'b',L2*100,z2b*100,'r','Linewidth',2);
set(gca,'Fontsize',12,'Fontweight','bold')
xlim([0 Inf])
xlabel('Cavity Length L (cm)','Fontsize',12)
ylabel('Distances, z_1, z_2 (cm)','Fontsize',12)
legend([ih1 ih2],{'z_1','z_2'},'Location','Best')
legend('boxoff')
title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
hold off
grid on 

ip_L1 = find(L1>0);
ip_L2 = find(L2>0);

if isempty(ip_L1) & isempty(ip_L2)
   disp('-----  Check radius R1 and R2     ------')
   disp('----- Always Unstable Resonator   ------')
   disp('-----      Not usable L !         ------')
end


else
figure(nfig)

plot(L*100,w0*10^6,'Linewidth',2)
set(gca,'Fontsize',12,'Fontweight','bold')
xlim([0 Inf])
xlabel('Cavity Length L (cm)','Fontsize',12)
ylabel('Minimum Waist, w_0 (\mum)','Fontsize',12)
title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
grid on 

nfig = nfig + 1;
figure(nfig)

plot(L*100,z0*100,'Linewidth',2)
set(gca,'Fontsize',12,'Fontweight','bold')
xlim([0 Inf])
xlabel('Cavity Length L (cm)','Fontsize',12)
ylabel('Rayleigh Distance, z_0 (cm)','Fontsize',12)
title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
grid on 

nfig = nfig + 1;
figure(nfig)

plot(L*100,g12,'Linewidth',2)
set(gca,'Fontsize',12,'Fontweight','bold')
xlim([0 Inf])
xlabel('Cavity Length L (cm)','Fontsize',12)
ylabel('Stability Term, g_1g_2','Fontsize',12)
title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
grid on 

nfig = nfig + 1;
figure(nfig)

plot(L*100,z1*100,L*100,z2*100,'Linewidth',2)
set(gca,'Fontsize',12,'Fontweight','bold')
xlim([0 Inf])
xlabel('Cavity Length L (cm)','Fontsize',12)
ylabel('Distances, z_1, z_2 (cm)','Fontsize',12)
legend('z_1','z_2','Location','Best')
legend('boxoff')
title([' R_1 = ',num2str(R1),'m, R_2 = ',num2str(R2),'m, \lambda_0 = ',num2str(lambda0),'\mum'],'Fontsize',11)
grid on 

ip_L = find(L>0);

if isempty(ip_L)
   disp('-----  Check radius R1 and R2     ------')
   disp('----- Always Unstable Resonator   ------')
   disp('-----      Not usable L !         ------')
end


end
